﻿' The channelData class holds the relevant information for an A/D channel. It holds both
' the options as the current measuring statistics.
'
'

Public Class channelData

    ' Definition of the private variables
    Private _name As String = ""
    Private _enable As Boolean = True
    Private _enableAttenuation As Boolean = True
    Private _Rext1 As Double = 100
    Private _Rext2 As Double = 100
    Private _min As Double = 0
    Private _max As Double = 0
    Private _avg As Double = 0
    Private _attenuationMode As Integer = attenuationSetting.NO_ATTENUATION
    Private _bypassResistor As Boolean

    Public rawAVGtotal As Integer = 0
    Public AVGcounter As Integer = 0
    Public ADvalue As Integer = 0
    Public voltsCur As Double = 0
    Public voltsMin As Double = 0
    Public voltsMax As Double = 0


    ''' <summary>
    ''' This enumeration holds all possible values for the attenuation setting.
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum attenuationSetting
        NO_ATTENUATION = 1
        SINGLE_RESISTOR = 2
        DOUBLE_RESISTOR = 3
    End Enum


    ''' <summary>
    ''' The property attenuationMode holds the desired attenuation mode. Possibilities:
    '''   + NO_ATTENUATION
    '''   + SINGLE_RESISTOR
    '''   + DOUBLE_RESISTOR
    ''' </summary>
    ''' <value>The desired attenuation mode</value>
    ''' <returns>The selected attenuation mode</returns>
    ''' <remarks></remarks>
    Public Property attenuationMode As attenuationSetting
        Get
            Return _attenuationMode
        End Get
        Set(ByVal value As attenuationSetting)
            _attenuationMode = value
        End Set
    End Property



    ''' <summary>
    ''' The property bypassResistor holds if the onboard R3/R4 resistor is bypassed or not. This can be
    ''' done to increase the accuracy of the K8055's DAC convertor.
    ''' </summary>
    ''' <value>A boolean indicating if you want to disable the external resistor</value>
    ''' <returns>The current mode</returns>
    ''' <remarks></remarks>
    Public Property bypassResistor As Boolean
        Get
            Return _bypassResistor
        End Get
        Set(ByVal value As Boolean)
            _bypassResistor = value
        End Set
    End Property



    ''' <summary>
    ''' The constructor method for the class
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        name = ""
        enable = True
        Rext1 = 0
        Rext2 = 0
    End Sub


    ''' <summary>
    ''' The name property allows you to set the custom name for the channel
    ''' </summary>
    ''' <value>A String containing the desired name</value>
    ''' <returns>A String containing the set name</returns>
    ''' <remarks></remarks>
    Public Property name As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            _name = value
        End Set
    End Property


    ''' <summary>
    ''' The enable property allows you to specify if the channel is enabled
    ''' </summary>
    ''' <value>A boolean indicating if the channel is enabled or not</value>
    ''' <returns>The current status of the enable property</returns>
    ''' <remarks></remarks>
    Public Property enable As Boolean
        Get
            Return _enable
        End Get
        Set(ByVal value As Boolean)
            _enable = value
        End Set
    End Property



    ''' <summary>
    ''' The Rext1 property holds the first external resistor. This is always the resistor between the
    ''' Voltage (in) and the analog input of the K8055. This resistor is always expressed in kOhm.
    ''' </summary>
    ''' <value>The desired value for Rext1</value>
    ''' <returns>The current value for Rext1</returns>
    ''' <remarks></remarks>
    Public Property Rext1 As Double
        Get
            Return _Rext1
        End Get
        Set(ByVal value As Double)
            _Rext1 = value
        End Set
    End Property



    ''' <summary>
    ''' The Rext2 property holds the first external resistor. This is always the resistor between the
    ''' the analog input of the K8055 and the ground. This resistor is always expressed in kOhm.
    ''' </summary>
    ''' <value>The desired value for Rext2</value>
    ''' <returns>The current value for Rext2</returns>
    ''' <remarks></remarks>
    Public Property Rext2 As Double
        Get
            Return _Rext2
        End Get
        Set(ByVal value As Double)
            _Rext2 = value
        End Set
    End Property



    ''' <summary>
    ''' The function convertValue will rescale a given voltage to the correct voltage, by calculating
    ''' the resistors.
    ''' </summary>
    ''' <param name="voltage"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function convertValue(ByVal voltage As Double) As Double

        Dim Rint As Double
        Dim response As Double


        If bypassResistor = True Then
            Rint = series(100, 0)
        Else
            Rint = series(100, 1)
        End If


        Select Case attenuationMode
            Case attenuationSetting.DOUBLE_RESISTOR
                ' Network with 2 external resistors
                response = voltage * (series(Rext1, parallel(Rext2, Rint)) / parallel(Rext2, Rint))

            Case attenuationSetting.NO_ATTENUATION
                ' Network without external resistor
                response = voltage

            Case attenuationSetting.SINGLE_RESISTOR
                ' Network with a single resistor
                response = voltage * (series(Rext1, Rint) / Rint)
        End Select

        Return response
    End Function



    ''' <summary>
    ''' parallel will calculate the equivalent parallel resistor for the given values
    ''' </summary>
    ''' <param name="R1">The value of the first resistor</param>
    ''' <param name="R2">The value of the second resistor</param>
    ''' <returns>The equivalent parallel resistor</returns>
    ''' <remarks></remarks>
    Private Function parallel(ByVal R1 As Double, ByVal R2 As Double) As Double
        Return (R1 * R2) / (R1 + R2)
    End Function



    ''' <summary>
    ''' parallel will calculate the equivalent series resistor for the given values
    ''' </summary>
    ''' <param name="R1">The value of the first resistor</param>
    ''' <param name="R2">The value of the second resistor</param>
    ''' <returns>The equivalent series resistor</returns>
    ''' <remarks></remarks>
    Private Function series(ByVal R1 As Double, ByVal R2 As Double) As Double
        Return (R1 + R2)
    End Function



    ''' <summary>
    ''' The min property indicates the minimum measured voltage.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property min As Double
        Get
            Return _min
        End Get
        Set(ByVal value As Double)
            _min = value
        End Set
    End Property



    ''' <summary>
    ''' The max property indicates the minimum measured voltage.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property max As Double
        Get
            Return _max
        End Get
        Set(ByVal value As Double)
            _max = value
        End Set
    End Property
End Class