﻿Public Class frmChanConfig

    ' Declarations
    Private currentChannelConfig As channelData       ' Current configuration that is loaded into the object
    Private closedByOk As Boolean = False             ' Is the object closed by the OK key?



    ''' <summary>
    ''' The currentConfig property allows to set/get the current channel configuration object.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property currentConfig As channelData
        Get
            Return currentChannelConfig
        End Get
        Set(ByVal value As channelData)
            currentChannelConfig = value
        End Set
    End Property



    ''' <summary>
    ''' cmdOk_Click is launched when the user pressed OK to close the dialog Window.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOk.Click
        With currentConfig
            Dim rExt1 As Double
            Dim rExt2 As Double

            If inputValuesAreValid(rExt1, Rext2) Then
                .Rext1 = rExt1
                .Rext2 = rExt2
                .bypassResistor = chkBypassInternalResistor.Checked
                .attenuationMode = getSelectedAttenuationMethod()
                .enable = chkEnable.Checked
                .name = txtDescription.Text
            Else
                messageBox("Invalid resistor values: no numbers. Please verify.")
                Exit Sub
            End If
        End With

        DialogResult = DialogResult.OK
        Close()
    End Sub



    ''' <summary>
    ''' cmdCancel_Click is launched when the user presses the Cancel dialog Window.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
        Close()
    End Sub



    ''' <summary>
    ''' inpRext_TextChanged is launched when one of the textboxes for the external resistor is modified. It will
    ''' immediately change all the labels and calculations to the new resistance.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub inpRext_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles inpRext1.TextChanged, InpRext2.TextChanged, chkBypassInternalResistor.Click, RAD_EXTERNAL_1RES.Click, RAD_EXTERNAL_2RESISTORS.Click, RAD_NOATTENUATION.Click
        ' here we will calculate all the necessary values
        refreshFields()
    End Sub



    ''' <summary>
    ''' refreshFields() will recalculate all values for the resistor network.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub refreshFields()
        Dim Rext1 As Double
        Dim Rext2 As Double
        Dim Rint As Double


        Dim Umax As Double
        Dim resolution As Double
        Dim attenuation As Double


        ' Calculate the effect of the bypass resistor
        If chkBypassInternalResistor.Checked Then
            Rint = series(100, 0)
        Else
            Rint = series(100, 1)
        End If

        If inputValuesAreValid(Rext1, Rext2) Then
            ' Set initial values for the resistors
            imgRext1.Text = Rext1 & "k"
            imgRext2.Text = Rext2 & "k"

            Select Case getSelectedAttenuationMethod()
                Case channelData.attenuationSetting.SINGLE_RESISTOR
                    Umax = (5 * series(Rext1, Rint)) / Rint
                    resolution = Umax / 255
                    attenuation = Umax / 5
                    imgRext2.Text = "(not present)"

                Case channelData.attenuationSetting.DOUBLE_RESISTOR
                    Rint = parallel(Rint, Rext2)
                    Umax = (5 * series(Rext1, Rint)) / Rint
                    resolution = Umax / 255
                    attenuation = Umax / 5

                Case channelData.attenuationSetting.NO_ATTENUATION
                    Umax = 5
                    resolution = Umax / 255
                    attenuation = 5
                    imgRext1.Text = "(not present: bypassed!)"
                    imgRext2.Text = "(not present)"
            End Select

            lblMaxVoltage.Text = Format(Umax, "0.00") & " Volt"
            lblResolution.Text = Format(resolution, "0.0000") & " Volt/step"
            lblAttenuation.Text = Format(attenuation, "0.00") & " x"

        Else
            ' Invalid data
        End If


    End Sub


    ''' <summary>
    ''' inputValuesAreValid will determine if the input values are valid numbers (Double format). Only then
    ''' the function will return true as the result.
    ''' 
    ''' The results are given back through the Rext1 and Rext2 resistors.
    ''' </summary>
    ''' <param name="Rext1">The value of the first external resistor</param>
    ''' <param name="Rext2">The value of the second external resistor</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function inputValuesAreValid(ByRef Rext1 As Double, ByRef Rext2 As Double) As Boolean
        Dim validFields As Boolean = True

        EP.Clear()

        If Double.TryParse(inpRext1.Text, Rext1) = False Then
            validFields = False
            EP.SetError(lblK1, "Invalid value")
        End If

        If Double.TryParse(InpRext2.Text, Rext2) = False Then
            validFields = False
            EP.SetError(lblK2, "Invalid value")
        End If

        Return validFields
    End Function



    ''' <summary>
    ''' parallel will calculate the equivalent parallel resistor for the given values
    ''' </summary>
    ''' <param name="R1">The value of the first resistor</param>
    ''' <param name="R2">The value of the second resistor</param>
    ''' <returns>The equivalent parallel resistor</returns>
    ''' <remarks></remarks>
    Private Function parallel(ByVal R1 As Double, ByVal R2 As Double) As Double
        Return (R1 * R2) / (R1 + R2)
    End Function



    ''' <summary>
    ''' parallel will calculate the equivalent series resistor for the given values
    ''' </summary>
    ''' <param name="R1">The value of the first resistor</param>
    ''' <param name="R2">The value of the second resistor</param>
    ''' <returns>The equivalent series resistor</returns>
    ''' <remarks></remarks>
    Private Function series(ByVal R1 As Double, ByVal R2 As Double) As Double
        Return (R1 + R2)
    End Function




    ''' <summary>
    ''' getSelectedAttenuationMethod will read the radio buttons and return the selected (enum) option.
    ''' </summary>
    ''' <returns>The number of the option</returns>
    ''' <remarks></remarks>
    Private Function getSelectedAttenuationMethod() As channelData.attenuationSetting
        If RAD_EXTERNAL_1RES.Checked Then Return channelData.attenuationSetting.SINGLE_RESISTOR
        If RAD_EXTERNAL_2RESISTORS.Checked Then Return channelData.attenuationSetting.DOUBLE_RESISTOR
        Return channelData.attenuationSetting.NO_ATTENUATION
    End Function



    ''' <summary>
    ''' RAD_NOATTENUATION_CheckedChanged occurs when the radio button is selected. The unwanted resistors fields will be disabled
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub RAD_NOATTENUATION_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RAD_NOATTENUATION.CheckedChanged
        inpRext1.Enabled = False
        InpRext2.Enabled = False
    End Sub



    ''' <summary>
    ''' RAD_EXTERNAL_2RESISTORS_CheckedChanged occurs when the radio button is selected. The unwanted resistors fields will be disabled
    ''' </summary>
    Private Sub RAD_EXTERNAL_2RESISTORS_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RAD_EXTERNAL_2RESISTORS.CheckedChanged
        inpRext1.Enabled = True
        InpRext2.Enabled = True
    End Sub



    ''' <summary>
    ''' RAD_EXTERNAL_1RES_CheckedChanged occurs when the radio button is selected. The unwanted resistors fields will be disabled
    ''' </summary>
    Private Sub RAD_EXTERNAL_1RES_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RAD_EXTERNAL_1RES.CheckedChanged
        inpRext1.Enabled = True
        InpRext2.Enabled = False
    End Sub


    ''' <summary>
    ''' ShowDialog() is called when the user opens the form
    ''' </summary>
    ''' <returns>The dialogresult (OK or Cancel)</returns>
    ''' <remarks></remarks>
    Public Overloads Function ShowDialog() As Windows.Forms.DialogResult
        'load up order defaults
        DialogResult = DialogResult.Cancel

        If Not (currentChannelConfig Is Nothing) Then
            With currentChannelConfig
                chkEnable.Checked = .enable
                txtDescription.Text = .name
                inpRext1.Text = .Rext1
                InpRext2.Text = .Rext2
                chkBypassInternalResistor.Checked = .bypassResistor

                Select Case .attenuationMode
                    Case channelData.attenuationSetting.DOUBLE_RESISTOR
                        RAD_EXTERNAL_2RESISTORS.Checked = True
                    Case channelData.attenuationSetting.NO_ATTENUATION
                        RAD_NOATTENUATION.Checked = True
                    Case channelData.attenuationSetting.SINGLE_RESISTOR
                        RAD_EXTERNAL_1RES.Checked = True
                    Case Else
                        RAD_NOATTENUATION.Checked = True
                End Select
            End With

            refreshFields()
        Else
            ' This should never occur
        End If

        Return MyBase.ShowDialog()
    End Function
End Class